/// @file substrhuff.hpp
/// Compressor that uses the SubString Parser, a 0-Order Model and a Huffman Coder
///
/// @remarks
/// This compressor basically does nothing but coordinate these three components.
///
/// @author mouser
/// @date   2003.07.29
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// recursive header protection
#ifndef SubStrHuffCompressorH
#define SubStrHuffCompressorH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// application includes
#include "../compressor_statistical.hpp"
#include "../../coders/coder_huffman/huffmancoder.hpp"
#include "../../parsers/parser_substring/substringparser.hpp"
#include "../../modelers/modeler_zeroorder/zeroorder_modeler.hpp"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
/// The SubStrHuff Compressor is a basic Statistical compressor, that
/// instantiates a Zero-order modeler, with a huffman coder, and a SubString
/// parser.
///
/// @ingroup Compressors
///
class SubStrHuffCompressor : public OctaneCompressor_Statistical
{
public:
	/// constructor
	SubStrHuffCompressor(bool registerme=false);
	/// destructor
	~SubStrHuffCompressor() {;};
public:
	//---------------------------------------------------------------------------
	// OCTANE PUBLIC API - RTTI AND HELP FUNCTIONS - these are supported by all derived classes
	virtual std::string GetClassName() {return "SubStrHuff";}
	virtual std::string GetDescription() {return "SubString Huffman Compressor";}
	//---------------------------------------------------------------------------
public:
	//---------------------------------------------------------------------------
	// COMPRESSOR PUBLIC API - FACTORY FUNCTION / MAKE AN INSTANTIATION OF DERIVED COMPRESSOR
	virtual OctaneCompressor *MakeCompressorInstantiation() {return new SubStrHuffCompressor();};
	//---------------------------------------------------------------------------
};
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
